@@ -138,7 +138,7 @@ H5G__stab_create_components(H5F_t *f, H5O_stab_t *stab, size_t size_hint)
 
     /* Pin the heap down in memory */
     if (NULL == (heap = H5HL_protect(f, stab->heap_addr, H5AC__NO_FLAGS_SET)))
-        HGOTO_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to protect symbol table heap");
+        HGOTO_ERROR(H5E_SYM, H5E_CANTPROTECT, FAIL, "unable to protect symbol table heap");
 
     /* Insert name into the heap */
     if (H5HL_insert(f, heap, (size_t)1, "", &name_offset) < 0)
@@ -152,7 +152,7 @@ H5G__stab_create_components(H5F_t *f, H5O_stab_t *stab, size_t size_hint)
 done:
     /* Release resources */
     if (heap && FAIL == H5HL_unprotect(heap))
-        HDONE_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to unprotect symbol table heap");
+        HDONE_ERROR(H5E_SYM, H5E_CANTUNPROTECT, FAIL, "unable to unprotect symbol table heap");
 
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5G__stab_create_components() */
@@ -227,8 +227,8 @@ done:
  *-------------------------------------------------------------------------
  */
 herr_t
-H5G__stab_insert_real(H5F_t *f, const H5O_stab_t *stab, const char *name, H5O_link_t *obj_lnk,
-                      H5O_type_t obj_type, const void *crt_info)
+H5G__stab_insert_real(H5F_t *f, const H5O_stab_t *stab, H5O_link_t *obj_lnk, H5O_type_t obj_type,
+                      const void *crt_info)
 {
     H5HL_t      *heap = NULL;         /* Pointer to local heap */
     H5G_bt_ins_t udata;               /* Data to pass through B-tree	*/
@@ -239,19 +239,19 @@ H5G__stab_insert_real(H5F_t *f, const H5O_stab_t *stab, const char *name, H5O_li
     /* check arguments */
     assert(f);
     assert(stab);
-    assert(name && *name);
     assert(obj_lnk);
 
     /* Pin the heap down in memory */
     if (NULL == (heap = H5HL_protect(f, stab->heap_addr, H5AC__NO_FLAGS_SET)))
-        HGOTO_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to protect symbol table heap");
+        HGOTO_ERROR(H5E_SYM, H5E_CANTPROTECT, FAIL, "unable to protect symbol table heap");
 
     /* Initialize data to pass through B-tree */
-    udata.common.name = name;
-    udata.common.heap = heap;
-    udata.lnk         = obj_lnk;
-    udata.obj_type    = obj_type;
-    udata.crt_info    = crt_info;
+    udata.common.name       = obj_lnk->name;
+    udata.common.heap       = heap;
+    udata.common.block_size = H5HL_heap_get_size(heap);
+    udata.lnk               = obj_lnk;
+    udata.obj_type          = obj_type;
+    udata.crt_info          = crt_info;
 
     /* Insert into symbol table */
     if (H5B_insert(f, H5B_SNODE, stab->btree_addr, &udata) < 0)
@@ -260,7 +260,7 @@ H5G__stab_insert_real(H5F_t *f, const H5O_stab_t *stab, const char *name, H5O_li
 done:
     /* Release resources */
     if (heap && H5HL_unprotect(heap) < 0)
-        HDONE_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to unprotect symbol table heap");
+        HDONE_ERROR(H5E_SYM, H5E_CANTUNPROTECT, FAIL, "unable to unprotect symbol table heap");
 
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5G__stab_insert_real() */
@@ -268,17 +268,14 @@ done:
 /*-------------------------------------------------------------------------
  * Function:	H5G__stab_insert
  *
- * Purpose:	Insert a new symbol into the table described by GRP_ENT in
- *		file F.	 The name of the new symbol is NAME and its symbol
- *		table entry is OBJ_ENT.
+ * Purpose:	Insert a new link, OBJ_LNK, into the group, GRP_OLOC.
  *
  * Return:	Non-negative on success/Negative on failure
  *
  *-------------------------------------------------------------------------
  */
 herr_t
-H5G__stab_insert(const H5O_loc_t *grp_oloc, const char *name, H5O_link_t *obj_lnk, H5O_type_t obj_type,
-                 const void *crt_info)
+H5G__stab_insert(const H5O_loc_t *grp_oloc, H5O_link_t *obj_lnk, H5O_type_t obj_type, const void *crt_info)
 {
     H5O_stab_t stab;                /* Symbol table message		*/
     herr_t     ret_value = SUCCEED; /* Return value */
@@ -287,15 +284,14 @@ H5G__stab_insert(const H5O_loc_t *grp_oloc, const char *name, H5O_link_t *obj_ln
 
     /* check arguments */
     assert(grp_oloc && grp_oloc->file);
-    assert(name && *name);
     assert(obj_lnk);
 
     /* Retrieve symbol table message */
     if (NULL == H5O_msg_read(grp_oloc, H5O_STAB_ID, &stab))
-        HGOTO_ERROR(H5E_SYM, H5E_BADMESG, FAIL, "not a symbol table");
+        HGOTO_ERROR(H5E_SYM, H5E_CANTGET, FAIL, "not a symbol table");
 
-    if (H5G__stab_insert_real(grp_oloc->file, &stab, name, obj_lnk, obj_type, crt_info) < 0)
-        HGOTO_ERROR(H5E_DATATYPE, H5E_CANTINIT, H5_ITER_ERROR, "unable to insert the name");
+    if (H5G__stab_insert_real(grp_oloc->file, &stab, obj_lnk, obj_type, crt_info) < 0)
+        HGOTO_ERROR(H5E_SYM, H5E_CANTINSERT, H5_ITER_ERROR, "unable to insert the link");
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
@@ -325,25 +321,26 @@ H5G__stab_remove(const H5O_loc_t *loc, H5RS_str_t *grp_full_path_r, const char *
 
     /* Read in symbol table message */
     if (NULL == H5O_msg_read(loc, H5O_STAB_ID, &stab))
-        HGOTO_ERROR(H5E_SYM, H5E_BADMESG, FAIL, "not a symbol table");
+        HGOTO_ERROR(H5E_SYM, H5E_CANTGET, FAIL, "not a symbol table");
 
     /* Pin the heap down in memory */
     if (NULL == (heap = H5HL_protect(loc->file, stab.heap_addr, H5AC__NO_FLAGS_SET)))
-        HGOTO_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to protect symbol table heap");
+        HGOTO_ERROR(H5E_SYM, H5E_CANTPROTECT, FAIL, "unable to protect symbol table heap");
 
     /* Initialize data to pass through B-tree */
-    udata.common.name     = name;
-    udata.common.heap     = heap;
-    udata.grp_full_path_r = grp_full_path_r;
+    udata.common.name       = name;
+    udata.common.heap       = heap;
+    udata.common.block_size = H5HL_heap_get_size(heap);
+    udata.grp_full_path_r   = grp_full_path_r;
 
     /* Remove from symbol table */
     if (H5B_remove(loc->file, H5B_SNODE, stab.btree_addr, &udata) < 0)
-        HGOTO_ERROR(H5E_SYM, H5E_CANTINIT, FAIL, "unable to remove entry");
+        HGOTO_ERROR(H5E_SYM, H5E_CANTREMOVE, FAIL, "unable to remove entry");
 
 done:
     /* Release resources */
     if (heap && H5HL_unprotect(heap) < 0)
-        HDONE_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to unprotect symbol table heap");
+        HDONE_ERROR(H5E_SYM, H5E_CANTUNPROTECT, FAIL, "unable to unprotect symbol table heap");
 
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5G__stab_remove() */
@@ -379,25 +376,26 @@ H5G__stab_remove_by_idx(const H5O_loc_t *grp_oloc, H5RS_str_t *grp_full_path_r,
 
     /* Read in symbol table message */
     if (NULL == H5O_msg_read(grp_oloc, H5O_STAB_ID, &stab))
-        HGOTO_ERROR(H5E_SYM, H5E_BADMESG, FAIL, "not a symbol table");
+        HGOTO_ERROR(H5E_SYM, H5E_CANTGET, FAIL, "not a symbol table");
 
     /* Pin the heap down in memory */
     if (NULL == (heap = H5HL_protect(grp_oloc->file, stab.heap_addr, H5AC__NO_FLAGS_SET)))
-        HGOTO_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to protect symbol table heap");
+        HGOTO_ERROR(H5E_SYM, H5E_CANTPROTECT, FAIL, "unable to protect symbol table heap");
 
     /* Initialize data to pass through B-tree */
-    udata.common.name     = obj_lnk.name;
-    udata.common.heap     = heap;
-    udata.grp_full_path_r = grp_full_path_r;
+    udata.common.name       = obj_lnk.name;
+    udata.common.heap       = heap;
+    udata.common.block_size = H5HL_heap_get_size(heap);
+    udata.grp_full_path_r   = grp_full_path_r;
 
     /* Remove link from symbol table */
     if (H5B_remove(grp_oloc->file, H5B_SNODE, stab.btree_addr, &udata) < 0)
-        HGOTO_ERROR(H5E_SYM, H5E_CANTINIT, FAIL, "unable to remove entry");
+        HGOTO_ERROR(H5E_SYM, H5E_CANTREMOVE, FAIL, "unable to remove entry");
 
 done:
     /* Release resources */
     if (heap && H5HL_unprotect(heap) < 0)
-        HDONE_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to unprotect symbol table heap");
+        HDONE_ERROR(H5E_SYM, H5E_CANTUNPROTECT, FAIL, "unable to unprotect symbol table heap");
 
     /* Reset the link information, if we have a copy */
     if (lnk_copied)
@@ -431,7 +429,7 @@ H5G__stab_delete(H5F_t *f, const H5O_stab_t *stab)
 
     /* Pin the heap down in memory */
     if (NULL == (heap = H5HL_protect(f, stab->heap_addr, H5AC__NO_FLAGS_SET)))
-        HGOTO_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to protect symbol table heap");
+        HGOTO_ERROR(H5E_SYM, H5E_CANTPROTECT, FAIL, "unable to protect symbol table heap");
 
     /* Set up user data for B-tree deletion */
     udata.common.name = NULL;
@@ -443,7 +441,7 @@ H5G__stab_delete(H5F_t *f, const H5O_stab_t *stab)
 
     /* Release resources */
     if (H5HL_unprotect(heap) < 0)
-        HGOTO_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to unprotect symbol table heap");
+        HGOTO_ERROR(H5E_SYM, H5E_CANTUNPROTECT, FAIL, "unable to unprotect symbol table heap");
     heap = NULL;
 
     /* Delete local heap for names */
@@ -453,7 +451,7 @@ H5G__stab_delete(H5F_t *f, const H5O_stab_t *stab)
 done:
     /* Release resources */
     if (heap && H5HL_unprotect(heap) < 0)
-        HDONE_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to unprotect symbol table heap");
+        HDONE_ERROR(H5E_SYM, H5E_CANTUNPROTECT, FAIL, "unable to unprotect symbol table heap");
 
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5G__stab_delete() */
@@ -484,11 +482,11 @@ H5G__stab_iterate(const H5O_loc_t *oloc, H5_iter_order_t order, hsize_t skip, hs
 
     /* Get the B-tree info */
     if (NULL == H5O_msg_read(oloc, H5O_STAB_ID, &stab))
-        HGOTO_ERROR(H5E_SYM, H5E_NOTFOUND, FAIL, "unable to determine local heap address");
+        HGOTO_ERROR(H5E_SYM, H5E_CANTGET, FAIL, "unable to determine local heap address");
 
     /* Pin the heap down in memory */
     if (NULL == (heap = H5HL_protect(oloc->file, stab.heap_addr, H5AC__READ_ONLY_FLAG)))
-        HGOTO_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to protect symbol table heap");
+        HGOTO_ERROR(H5E_SYM, H5E_CANTPROTECT, FAIL, "unable to protect symbol table heap");
 
     /* Check on iteration order */
     /* ("native" iteration order is increasing for this link storage mechanism) */
@@ -521,7 +519,7 @@ H5G__stab_iterate(const H5O_loc_t *oloc, H5_iter_order_t order, hsize_t skip, hs
 
         /* Iterate over the group members */
         if (H5B_iterate(oloc->file, H5B_SNODE, stab.btree_addr, H5G__node_build_table, &udata) < 0)
-            HGOTO_ERROR(H5E_SYM, H5E_NOTFOUND, FAIL, "unable to build link table");
+            HGOTO_ERROR(H5E_SYM, H5E_BADITER, FAIL, "unable to build link table");
 
         /* Check for skipping out of bounds */
         if (skip > 0 && (size_t)skip >= ltable.nlinks)
@@ -539,9 +537,9 @@ H5G__stab_iterate(const H5O_loc_t *oloc, H5_iter_order_t order, hsize_t skip, hs
 done:
     /* Release resources */
     if (heap && H5HL_unprotect(heap) < 0)
-        HDONE_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to unprotect symbol table heap");
+        HDONE_ERROR(H5E_SYM, H5E_CANTUNPROTECT, FAIL, "unable to unprotect symbol table heap");
     if (ltable.lnks && H5G__link_release_table(&ltable) < 0)
-        HDONE_ERROR(H5E_SYM, H5E_CANTFREE, FAIL, "unable to release link table");
+        HDONE_ERROR(H5E_SYM, H5E_CANTRELEASE, FAIL, "unable to release link table");
 
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5G__stab_iterate() */
@@ -572,11 +570,11 @@ H5G__stab_count(const H5O_loc_t *oloc, hsize_t *num_objs)
 
     /* Get the B-tree info */
     if (NULL == H5O_msg_read(oloc, H5O_STAB_ID, &stab))
-        HGOTO_ERROR(H5E_SYM, H5E_NOTFOUND, FAIL, "unable to determine local heap address");
+        HGOTO_ERROR(H5E_SYM, H5E_CANTGET, FAIL, "unable to determine local heap address");
 
     /* Iterate over the group members */
     if (H5B_iterate(oloc->file, H5B_SNODE, stab.btree_addr, H5G__node_sumup, num_objs) < 0)
-        HGOTO_ERROR(H5E_SYM, H5E_CANTINIT, FAIL, "iteration operator failed");
+        HGOTO_ERROR(H5E_SYM, H5E_BADITER, FAIL, "iteration operator failed");
 
 done:
     FUNC_LEAVE_NOAPI_TAG(ret_value)
@@ -610,14 +608,14 @@ H5G__stab_bh_size(H5F_t *f, const H5O_stab_t *stab, H5_ih_info_t *bh_info)
 
     /* Get the B-tree & symbol table node size info */
     if (H5B_get_info(f, H5B_SNODE, stab->btree_addr, &bt_info, H5G__node_iterate_size, &snode_size) < 0)
-        HGOTO_ERROR(H5E_BTREE, H5E_CANTINIT, FAIL, "iteration operator failed");
+        HGOTO_ERROR(H5E_SYM, H5E_CANTGET, FAIL, "iteration operator failed");
 
     /* Add symbol table & B-tree node sizes to index info */
     bh_info->index_size += snode_size + bt_info.size;
 
     /* Get the size of the local heap for the group */
     if (H5HL_heapsize(f, stab->heap_addr, &(bh_info->heap_size)) < 0)
-        HGOTO_ERROR(H5E_HEAP, H5E_CANTINIT, FAIL, "iteration operator failed");
+        HGOTO_ERROR(H5E_SYM, H5E_CANTGET, FAIL, "iteration operator failed");
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
@@ -640,6 +638,7 @@ H5G__stab_get_name_by_idx_cb(const H5G_entry_t *ent, void *_udata)
     H5G_bt_it_gnbi_t *udata = (H5G_bt_it_gnbi_t *)_udata;
     size_t            name_off;            /* Offset of name in heap */
     const char       *name;                /* Pointer to name string in heap */
+    size_t            block_size;          /* Size of the heap block */
     herr_t            ret_value = SUCCEED; /* Return value */
 
     FUNC_ENTER_PACKAGE
@@ -648,14 +647,17 @@ H5G__stab_get_name_by_idx_cb(const H5G_entry_t *ent, void *_udata)
     assert(ent);
     assert(udata && udata->heap);
 
+    /* Get the size of the heap block */
+    block_size = H5HL_heap_get_size(udata->heap);
+
     /* Get name offset in heap */
     name_off = ent->name_off;
 
     if ((name = (const char *)H5HL_offset_into(udata->heap, name_off)) == NULL)
         HGOTO_ERROR(H5E_SYM, H5E_CANTGET, FAIL, "unable to get symbol table link name");
 
-    if ((udata->name = H5MM_strdup(name)) == NULL)
-        HGOTO_ERROR(H5E_SYM, H5E_CANTGET, FAIL, "unable to duplicate symbol table link name");
+    if (NULL == (udata->name = H5MM_strndup(name, (block_size - name_off))))
+        HGOTO_ERROR(H5E_SYM, H5E_CANTCOPY, FAIL, "unable to duplicate symbol table link name");
 
 done:
     FUNC_LEAVE_NOAPI(ret_value)
@@ -690,11 +692,11 @@ H5G__stab_get_name_by_idx(const H5O_loc_t *oloc, H5_iter_order_t order, hsize_t
 
     /* Get the B-tree & local heap info */
     if (NULL == H5O_msg_read(oloc, H5O_STAB_ID, &stab))
-        HGOTO_ERROR(H5E_SYM, H5E_NOTFOUND, FAIL, "unable to determine local heap address");
+        HGOTO_ERROR(H5E_SYM, H5E_CANTGET, FAIL, "unable to determine local heap address");
 
     /* Pin the heap down in memory */
     if (NULL == (heap = H5HL_protect(oloc->file, stab.heap_addr, H5AC__READ_ONLY_FLAG)))
-        HGOTO_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to protect symbol table heap");
+        HGOTO_ERROR(H5E_SYM, H5E_CANTPROTECT, FAIL, "unable to protect symbol table heap");
 
     /* Remap index for decreasing iteration order */
     if (order == H5_ITER_DEC) {
@@ -702,7 +704,7 @@ H5G__stab_get_name_by_idx(const H5O_loc_t *oloc, H5_iter_order_t order, hsize_t
 
         /* Iterate over the symbol table nodes, to count the links */
         if (H5B_iterate(oloc->file, H5B_SNODE, stab.btree_addr, H5G__node_sumup, &nlinks) < 0)
-            HGOTO_ERROR(H5E_SYM, H5E_CANTINIT, FAIL, "iteration operator failed");
+            HGOTO_ERROR(H5E_SYM, H5E_BADITER, FAIL, "iteration operator failed");
 
         /* Map decreasing iteration order index to increasing iteration order index */
         n = nlinks - (n + 1);
@@ -718,11 +720,11 @@ H5G__stab_get_name_by_idx(const H5O_loc_t *oloc, H5_iter_order_t order, hsize_t
 
     /* Iterate over the group members */
     if (H5B_iterate(oloc->file, H5B_SNODE, stab.btree_addr, H5G__node_by_idx, &udata) < 0)
-        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "iteration operator failed");
+        HGOTO_ERROR(H5E_SYM, H5E_BADITER, FAIL, "iteration operator failed");
 
     /* If we don't know the name now, we almost certainly went out of bounds */
     if (udata.name == NULL)
-        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "index out of bound");
+        HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "index out of bound");
 
     /* Get the length of the name */
     *name_len = strlen(udata.name);
@@ -737,7 +739,7 @@ H5G__stab_get_name_by_idx(const H5O_loc_t *oloc, H5_iter_order_t order, hsize_t
 done:
     /* Release resources */
     if (heap && H5HL_unprotect(heap) < 0)
-        HDONE_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to unprotect symbol table heap");
+        HDONE_ERROR(H5E_SYM, H5E_CANTUNPROTECT, FAIL, "unable to unprotect symbol table heap");
 
     /* Free the duplicated name */
     if (udata_valid && udata.name != NULL)
@@ -768,7 +770,7 @@ H5G__stab_lookup_cb(const H5G_entry_t *ent, void *_udata)
     /* Check for setting link info */
     if (udata->lnk)
         /* Convert the entry to a link */
-        if (H5G__ent_to_link(udata->lnk, udata->heap, ent, udata->name) < 0)
+        if (H5G__ent_to_link(ent, udata->heap, udata->lnk) < 0)
             HGOTO_ERROR(H5E_SYM, H5E_CANTCONVERT, FAIL, "unable to convert symbol table entry to link");
 
 done:
@@ -803,11 +805,11 @@ H5G__stab_lookup(const H5O_loc_t *grp_oloc, const char *name, bool *found, H5O_l
 
     /* Retrieve the symbol table message for the group */
     if (NULL == H5O_msg_read(grp_oloc, H5O_STAB_ID, &stab))
-        HGOTO_ERROR(H5E_SYM, H5E_BADMESG, FAIL, "can't read message");
+        HGOTO_ERROR(H5E_SYM, H5E_CANTGET, FAIL, "can't read message");
 
     /* Pin the heap down in memory */
     if (NULL == (heap = H5HL_protect(grp_oloc->file, stab.heap_addr, H5AC__READ_ONLY_FLAG)))
-        HGOTO_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to protect symbol table heap");
+        HGOTO_ERROR(H5E_SYM, H5E_CANTPROTECT, FAIL, "unable to protect symbol table heap");
 
     /* Set up user data to pass to 'find' operation callback */
     udata.name = name;
@@ -815,10 +817,11 @@ H5G__stab_lookup(const H5O_loc_t *grp_oloc, const char *name, bool *found, H5O_l
     udata.heap = heap;
 
     /* Set up the user data for actual B-tree find operation */
-    bt_udata.common.name = name;
-    bt_udata.common.heap = heap;
-    bt_udata.op          = H5G__stab_lookup_cb;
-    bt_udata.op_data     = &udata;
+    bt_udata.common.name       = name;
+    bt_udata.common.heap       = heap;
+    bt_udata.common.block_size = H5HL_heap_get_size(heap);
+    bt_udata.op                = H5G__stab_lookup_cb;
+    bt_udata.op_data           = &udata;
 
     /* Search the B-tree */
     if (H5B_find(grp_oloc->file, H5B_SNODE, stab.btree_addr, found, &bt_udata) < 0)
@@ -827,7 +830,7 @@ H5G__stab_lookup(const H5O_loc_t *grp_oloc, const char *name, bool *found, H5O_l
 done:
     /* Release resources */
     if (heap && H5HL_unprotect(heap) < 0)
-        HDONE_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to unprotect symbol table heap");
+        HDONE_ERROR(H5E_SYM, H5E_CANTUNPROTECT, FAIL, "unable to unprotect symbol table heap");
 
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5G__stab_lookup() */
@@ -846,8 +849,7 @@ done:
 static herr_t
 H5G__stab_lookup_by_idx_cb(const H5G_entry_t *ent, void *_udata)
 {
-    H5G_bt_it_lbi_t *udata = (H5G_bt_it_lbi_t *)_udata;
-    const char      *name;                /* Pointer to name string in heap */
+    H5G_bt_it_lbi_t *udata     = (H5G_bt_it_lbi_t *)_udata;
     herr_t           ret_value = SUCCEED; /* Return value */
 
     FUNC_ENTER_PACKAGE
@@ -856,12 +858,8 @@ H5G__stab_lookup_by_idx_cb(const H5G_entry_t *ent, void *_udata)
     assert(ent);
     assert(udata && udata->heap);
 
-    /* Get a pointer to the link name */
-    if ((name = (const char *)H5HL_offset_into(udata->heap, ent->name_off)) == NULL)
-        HGOTO_ERROR(H5E_SYM, H5E_CANTGET, FAIL, "unable to get symbol table link name");
-
     /* Convert the entry to a link */
-    if (H5G__ent_to_link(udata->lnk, udata->heap, ent, name) < 0)
+    if (H5G__ent_to_link(ent, udata->heap, udata->lnk) < 0)
         HGOTO_ERROR(H5E_SYM, H5E_CANTCONVERT, FAIL, "unable to convert symbol table entry to link");
     udata->found = true;
 
@@ -894,11 +892,11 @@ H5G__stab_lookup_by_idx(const H5O_loc_t *grp_oloc, H5_iter_order_t order, hsize_
 
     /* Get the B-tree & local heap info */
     if (NULL == H5O_msg_read(grp_oloc, H5O_STAB_ID, &stab))
-        HGOTO_ERROR(H5E_SYM, H5E_NOTFOUND, FAIL, "unable to determine local heap address");
+        HGOTO_ERROR(H5E_SYM, H5E_CANTGET, FAIL, "unable to determine local heap address");
 
     /* Pin the heap down in memory */
     if (NULL == (heap = H5HL_protect(grp_oloc->file, stab.heap_addr, H5AC__READ_ONLY_FLAG)))
-        HGOTO_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to protect symbol table heap");
+        HGOTO_ERROR(H5E_SYM, H5E_CANTPROTECT, FAIL, "unable to protect symbol table heap");
 
     /* Remap index for decreasing iteration order */
     if (order == H5_ITER_DEC) {
@@ -906,7 +904,7 @@ H5G__stab_lookup_by_idx(const H5O_loc_t *grp_oloc, H5_iter_order_t order, hsize_
 
         /* Iterate over the symbol table nodes, to count the links */
         if (H5B_iterate(grp_oloc->file, H5B_SNODE, stab.btree_addr, H5G__node_sumup, &nlinks) < 0)
-            HGOTO_ERROR(H5E_SYM, H5E_CANTINIT, FAIL, "iteration operator failed");
+            HGOTO_ERROR(H5E_SYM, H5E_BADITER, FAIL, "iteration operator failed");
 
         /* Map decreasing iteration order index to increasing iteration order index */
         n = nlinks - (n + 1);
@@ -922,16 +920,16 @@ H5G__stab_lookup_by_idx(const H5O_loc_t *grp_oloc, H5_iter_order_t order, hsize_
 
     /* Iterate over the group members */
     if (H5B_iterate(grp_oloc->file, H5B_SNODE, stab.btree_addr, H5G__node_by_idx, &udata) < 0)
-        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "iteration operator failed");
+        HGOTO_ERROR(H5E_SYM, H5E_BADITER, FAIL, "iteration operator failed");
 
     /* If we didn't find the link, we almost certainly went out of bounds */
     if (!udata.found)
-        HGOTO_ERROR(H5E_ARGS, H5E_BADTYPE, FAIL, "index out of bound");
+        HGOTO_ERROR(H5E_ARGS, H5E_BADVALUE, FAIL, "index out of bound");
 
 done:
     /* Release resources */
     if (heap && H5HL_unprotect(heap) < 0)
-        HDONE_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to unprotect symbol table heap");
+        HDONE_ERROR(H5E_SYM, H5E_CANTUNPROTECT, FAIL, "unable to unprotect symbol table heap");
 
     FUNC_LEAVE_NOAPI(ret_value)
 } /* end H5G__stab_lookup_by_idx() */
@@ -941,8 +939,8 @@ done:
 /*-------------------------------------------------------------------------
  * Function:	H5G__stab_valid
  *
- * Purpose:	Verify that a group's symbol table message is valid.  If
- *              provided, the addresses in alt_stab will be tried if the
+ * Purpose:	Verify that a group's symbol table message is valid.
+ *              The addresses in alt_stab will be tried if the
  *              addresses in the group's stab message are invalid, and
  *              the stab message will be updated if necessary.
  *
@@ -960,22 +958,32 @@ herr_t
 H5G__stab_valid(H5O_loc_t *grp_oloc, H5O_stab_t *alt_stab)
 {
     H5O_stab_t stab;                /* Current symbol table */
-    H5HL_t    *heap      = NULL;    /* Pointer to local heap */
-    bool       changed   = false;   /* Whether stab has been modified */
+    H5HL_t    *heap    = NULL;      /* Pointer to local heap */
+    bool       changed = false;     /* Whether stab has been modified */
+    herr_t     bt_status;           /* B-tree status */
     herr_t     ret_value = SUCCEED; /* Return value */
 
     FUNC_ENTER_PACKAGE_TAG(grp_oloc->addr)
 
+    /* Sanity check */
+    assert(grp_oloc);
+    assert(alt_stab);
+
     /* Read the symbol table message */
     if (NULL == H5O_msg_read(grp_oloc, H5O_STAB_ID, &stab))
-        HGOTO_ERROR(H5E_SYM, H5E_BADMESG, FAIL, "unable to read symbol table message");
+        HGOTO_ERROR(H5E_SYM, H5E_CANTGET, FAIL, "unable to read symbol table message");
 
     /* Check if the symbol table message's b-tree address is valid */
-    if (H5B_valid(grp_oloc->file, H5B_SNODE, stab.btree_addr) < 0) {
-        /* Address is invalid, try the b-tree address in the alternate symbol
-         * table message */
-        if (!alt_stab || H5B_valid(grp_oloc->file, H5B_SNODE, alt_stab->btree_addr) < 0)
-            HGOTO_ERROR(H5E_BTREE, H5E_NOTFOUND, FAIL, "unable to locate b-tree");
+    H5E_PAUSE_ERRORS
+    {
+        bt_status = H5B_valid(grp_oloc->file, H5B_SNODE, stab.btree_addr);
+    }
+    H5E_RESUME_ERRORS
+
+    if (bt_status < 0) {
+        /* Address is invalid, try the b-tree address in the alternate symbol table message */
+        if (H5B_valid(grp_oloc->file, H5B_SNODE, alt_stab->btree_addr) < 0)
+            HGOTO_ERROR(H5E_SYM, H5E_BADVALUE, FAIL, "unable to locate b-tree");
         else {
             /* The alternate symbol table's b-tree address is valid.  Adjust the
              * symbol table message in the group. */
@@ -985,12 +993,16 @@ H5G__stab_valid(H5O_loc_t *grp_oloc, H5O_stab_t *alt_stab)
     }     /* end if */
 
     /* Check if the symbol table message's heap address is valid */
-    if (NULL == (heap = H5HL_protect(grp_oloc->file, stab.heap_addr, H5AC__READ_ONLY_FLAG))) {
-        /* Address is invalid, try the heap address in the alternate symbol
-         * table message */
-        if (!alt_stab ||
-            NULL == (heap = H5HL_protect(grp_oloc->file, alt_stab->heap_addr, H5AC__READ_ONLY_FLAG)))
-            HGOTO_ERROR(H5E_HEAP, H5E_NOTFOUND, FAIL, "unable to locate heap");
+    H5E_PAUSE_ERRORS
+    {
+        heap = H5HL_protect(grp_oloc->file, stab.heap_addr, H5AC__READ_ONLY_FLAG);
+    }
+    H5E_RESUME_ERRORS
+
+    if (NULL == heap) {
+        /* Address is invalid, try the heap address in the alternate symbol table message */
+        if (NULL == (heap = H5HL_protect(grp_oloc->file, alt_stab->heap_addr, H5AC__READ_ONLY_FLAG)))
+            HGOTO_ERROR(H5E_SYM, H5E_CANTPROTECT, FAIL, "unable to locate heap");
         else {
             /* The alternate symbol table's heap address is valid.  Adjust the
              * symbol table message in the group. */
@@ -1000,16 +1012,14 @@ H5G__stab_valid(H5O_loc_t *grp_oloc, H5O_stab_t *alt_stab)
     }     /* end if */
 
     /* Update the symbol table message and clear errors if necessary */
-    if (changed) {
-        H5E_clear_stack(NULL);
+    if (changed)
         if (H5O_msg_write(grp_oloc, H5O_STAB_ID, 0, H5O_UPDATE_TIME | H5O_UPDATE_FORCE, &stab) < 0)
-            HGOTO_ERROR(H5E_SYM, H5E_CANTINIT, FAIL, "unable to correct symbol table message");
-    } /* end if */
+            HGOTO_ERROR(H5E_SYM, H5E_CANTSET, FAIL, "unable to correct symbol table message");
 
 done:
     /* Release resources */
     if (heap && H5HL_unprotect(heap) < 0)
-        HDONE_ERROR(H5E_SYM, H5E_PROTECT, FAIL, "unable to unprotect symbol table heap");
+        HDONE_ERROR(H5E_SYM, H5E_CANTUNPROTECT, FAIL, "unable to unprotect symbol table heap");
 
     FUNC_LEAVE_NOAPI_TAG(ret_value)
 } /* end H5G__stab_valid */
